;;;;;; 'macros' for remote program evaluation ;;;;;;

; basic idea is that instead of writing:
; (run '((lookup host port "eval") (fun () (..what you want to do on the server..))
;
; you can just write:
; (run (on host port '..what you want to do..))
;
; on-port and on-default are shortcuts for standard values (localhost:8080)
;
; like macros, if you want to combine calls to the on function with other bits of program
; use quasi-quoting ` and unquote ,

;; wraps the program in a program that ships a thunk containing it to 
;; the given host/port
(define (on host port pgm)
  `((lookup ,host ,port "eval") (fun () ,pgm )))

;; on localhost:8080
(define (on-default pgm) (on-port "8080" pgm))
;; on localhost using given port
(define (on-port port pgm) (on "localhost" port pgm))
