;; Definition of structures for expressions and values.
;; all structures are serializable

(require scheme/serialize)


;; structure definitions for expressions
(define-serializable-struct expression ())

(define-serializable-struct (lit-exp   expression) (value))
(define-serializable-struct (var-exp   expression) (name))
(define-serializable-struct (if-exp    expression) (test then else))
(define-serializable-struct (and-exp   expression) (args))
(define-serializable-struct (or-exp    expression) (args))
(define-serializable-struct (begin-exp expression) (exps))
(define-serializable-struct (let-exp   expression) (params args body))
(define-serializable-struct (def-exp   expression) (id exp))
(define-serializable-struct (fun-exp   expression) (params body))
(define-serializable-struct (app-exp   expression) (fun args))
(define-serializable-struct (prim-exp  expression) (prim args))
(define-serializable-struct (set-exp   expression) (id exp))

;; reflective access to current jp
(define-serializable-struct (this-jp-exp  expression) ())

;; aspect deployment expressions
(define-serializable-struct (depl-on-exp  expression) (ds asp fun))
(define-serializable-struct (depl-exp     expression) (ds asp body))

;; values
(define-serializable-struct value ())
(define-serializable-struct (closure  value) (params body env aspects) #:mutable)
(define-serializable-struct (fun-stub value) (host name arg-count))

;; deployed aspects (deployment strategy components + pc/adv)
(define-serializable-struct dasp (c d f pc adv))
       

;;;; join points ;;;;
;; k is kind: 'new, 'call, 'exec
;; f is fun          (for new, this is either #f or the function of which the new function is a copy)
;; a is args         (for new, this is formal parameters)
;; p is parent jp  
;; h is current host  
;; th is target host (#f if not a remote call)
(define-serializable-struct jp (k f a p h th))

;; utility display and accessor functions for join points
(define (jp-string jp)
  (define (lookup-exported arg)
    (cond ((closure? arg)
           (let ((match (findf (lambda (elt) (eq? arg (cdr elt))) (get-exported-functions-list))))
             (if match
                 (format "<closure:~a>" (car match))
                 arg)))
           ((fun-stub? arg) (format "<stub:~a>"(fun-stub-name arg)))
           (else arg)))
  (format "[~a,f:~a,a:~a,p:~a,h:~a,th:~a]" (jp-k jp) (lookup-exported (jp-f jp)) (map lookup-exported (jp-a jp)) (jp-p jp) (jp-h jp) (jp-th jp)))

(define (jp-kind jp)
  (jp-k jp))

(define (jp-parent jp)
  (jp-p jp))

(define (jp-host jp)
  (jp-h jp))

(define (jp-target-host jp)
  (jp-th jp))

(define (jp-args jp)
  (jp-a jp))

(define (jp-fun jp)
  (jp-f jp))

(define (jp-depth jp)
  (if (not jp)
      0
      (if (not (jp-parent jp))
          1
          (+ 1 (jp-depth (jp-parent jp))))))
